@@ -12,14 +12,16 @@ from .eyepacs import Eyepacs
 from .drive import DRIVE
 from .rite import RITE
 from .idrid import IDRiD_Segmentation
+from .kimeye import KimEye
 # --> private dataset
 from .qualdr import QualDR, QualDR_Grading
 
 # Chest X-Ray
 from .chexpert import CheXpert, CheXpert_Small
+from .padchest import PadChest
 
 # Microscopy
 from .BBBC038v1_microscopy import BBBC038v1
 
-# Colposcopy
+# Cervix
 from .intel_mobileodt_cervical import IntelMobileODTCervical
