@@ -1,6 +1,8 @@
 import pandas as pd
+import os
 import torch as T
 import torchvision.transforms as tvt
+import cv2
 import numpy as np
 from os.path import join
 import PIL
@@ -58,7 +60,7 @@ class CheXpert(T.utils.data.Dataset):
                        # and some that are inferred from the filepaths:
                        'Patient', 'Study', 'View', 'Num Views',
                        # and the index that corresponds to __getitem__
-                       'index',
+                       'index', 'Path'
                        ]
     LABELS_ALL = LABELS_METADATA + LABELS_DIAGNOSTIC
 
@@ -75,7 +77,8 @@ class CheXpert(T.utils.data.Dataset):
         'Age': {i: i for i in range(91)}})
 
     @staticmethod
-    def format_labels(getitem_dct: dict, labels=LABELS_ALL, explode=False):
+    def format_labels(getitem_dct: dict, labels=LABELS_ALL,
+                      explode=False, as_tensor=False):
         """Helper method for converting the labels into a tensor or numpy array.
 
         :dct: the dict received in getitem_transform.
@@ -84,11 +87,16 @@ class CheXpert(T.utils.data.Dataset):
             'Enlarged Cardiomediastinum', 'Cardiomegaly', 'Lung Opacity', 'Lung
             Lesion', 'Edema', 'Consolidation', 'Pneumonia', 'Atelectasis',
             'Pneumothorax', 'Pleural Effusion', 'Pleural Other', 'Fracture',
-            'Support Devices'].
+            'Support Devices', 'Path'].
+            NOTE: If 'Path' in labels, also set `as_tensor=False`
         :explode: If true, convert each class into a one-hot vector.
           For instance, 'Age' would get expanded into a one-hot vector of 90
           ages.  The diagnostic classes are expanded to four values
           corresponding to [0 (neg), 1 (pos), 2 (uncertain), 3 (blank)].
+        :img_loader: one of {'cv2', 'pillow'} to choose whether to load images
+            with OpenCV or PIL.  OpenCV is a tiny bit faster and lower cpu overhead.
+        :as_tensor:  if True, return a torch.tensor, otherwise return numpy array.
+            Define as_tensor=False if 'Path' in `labels`
 
         :returns: If explode=False, return torch.tensor of numeric label values.
             If explode=True, return [torch.tensor(one_hot), ...] corresponding
@@ -102,20 +110,25 @@ class CheXpert(T.utils.data.Dataset):
                 tmp[getitem_dct['labels'][lname]] = 1
                 y.append(tmp)
         else:
-            y = T.tensor(getitem_dct['labels'][labels].values)
+            y = getitem_dct['labels'][labels]
+            if as_tensor:
+                y = T.tensor(y)
         return y
 
-    def __init__(self, dataset_dir="./data/CheXpert-v1.0/",
+    def __init__(self, dataset_dir=os.environ.get('chexpert_dir', "./data/CheXpert-v1.0/"),
                  use_train_set=True,
                  img_transform=tvt.ToTensor(),
                  getitem_transform=lambda x: (
                      x['image'], CheXpert.format_labels(x)),
-                 label_cleanup_dct=LABEL_CLEANUP_DICT
+                 label_cleanup_dct=LABEL_CLEANUP_DICT,
+                 img_loader='pillow'
                  ):
 
         train_or_valid = 'train' if use_train_set else 'valid'
         label_fp = f"{dataset_dir.rstrip('/')}/{train_or_valid}.csv"
         self.labels_csv = pd.read_csv(label_fp)
+        self.img_loader = img_loader
+        assert img_loader in {'pillow', 'cv2'}
 
         # add extra columns to the csv by parsing the filepath
         self.labels_csv = self.labels_csv.join(
@@ -143,13 +156,22 @@ class CheXpert(T.utils.data.Dataset):
             self.labels_csv.replace(label_cleanup_dct, inplace=True)
             self.labels_csv = self.labels_csv.astype('int')
 
+        # add filepath to images as a possible label
+        self.labels_csv['Path'] = self.labels_csv.index
+
     def __len__(self):
         return self.labels_csv.shape[0]
 
     def _getimage(self, index):
         fp = self.fps[index]
-        with PIL.Image.open(fp) as im:
-            im.load()
+        if self.img_loader == 'pillow':
+            with PIL.Image.open(fp) as im:
+                im.load()
+        elif self.img_loader == 'cv2':
+            im = cv2.imread(fp, cv2.IMREAD_GRAYSCALE | cv2.IMREAD_ANYDEPTH)
+        else:
+            raise NotImplementedError(self.img_loader)
+
         if self.__img_transform:
             im = self.__img_transform(im)
         dct = {'image': im, 'fp': fp}
@@ -190,7 +212,7 @@ class CheXpert(T.utils.data.Dataset):
 
 
 class CheXpert_Small(CheXpert):
-    def __init__(self, dataset_dir="./data/CheXpert-v1.0-small/", **kwargs):
+    def __init__(self, dataset_dir=os.environ.get('chexpert_dir', "./data/CheXpert-v1.0-small/"), **kwargs):
         super().__init__(dataset_dir=dataset_dir, **kwargs)
 
 
