@@ -4,12 +4,12 @@ Quick performance plots used as a script to visualize results.
 
 This is wonderfully useful for analyzing csv files of models!
 
-  - Expects a csv header with "epoch" as a column (customizable via --index-col)
+  - Expect a csv header with "epoch" as a column (customizable via --index-col)
   - Requires a directory structure:
       ./results/
                 run_id/perf.csv  # for mode 1 and 4
                       /log/perf*.csv  # for other modes
-      where: 
+      where:
           - ./results/ can be changed via  --data-dir
           - "run_id" is any directory name
           - "perf.csv" and "perf*.csv" can be changed  via --data-fp-regex
@@ -27,17 +27,16 @@ Usage:
 import argparse as ap
 import datetime as dt
 import IPython
-import sys
 import re
 import glob
 import os
-from os.path import join, exists, basename, dirname
-#  from os import exists
+from os.path import join, basename, dirname
 import pandas as pd
 import numpy as np
 import seaborn as sns
 from textwrap import dedent
 from matplotlib import pyplot as plt
+import warnings
 #  import mpld3
 #  import mpld3.plugins
 
@@ -60,8 +59,8 @@ def load_df_from_fp(fp, ns):
             df = pd.read_hdf(fp, ns.hdf_table)
         else:
             print('bug: file does not exist:', fp)
-    except:
-        print(f"   WARNING: failed to load {fp}")
+    except Exception as e:
+        print(f"   WARNING: failed to load {fp}.  {e}")
         df = pd.DataFrame()
     return df
 
@@ -69,7 +68,7 @@ def load_df_from_fp(fp, ns):
 def _mode1_get_frames(ns):
     for run_id in get_run_ids(ns):
         #  if not exists(join(dir, 'lock.finished')):
-            #  continue
+        #      continue
         dirp = f'{ns.data_dir}/{run_id}'
         gen = (
             ((run_id, fname), load_df_from_fp(join(dirp, fname), ns))
@@ -80,7 +79,8 @@ def _mode1_get_frames(ns):
 
 def _get_all_data(ns):
     cdfs_data = {}
-    timestamp = dt.datetime.utcnow().strftime('%Y%m%dT%H%M%S')  # date plot was created.  nothing to do with timestamp column.
+    # date the plot was created.  nothing to do with timestamp column.
+    #  timestamp = dt.datetime.utcnow().strftime('%Y%m%dT%H%M%S')
     for run_id in get_run_ids(ns):
         dirp = f'{ns.data_dir}/{run_id}/log'
         if not os.path.exists(dirp):
@@ -98,14 +98,14 @@ def _get_all_data(ns):
     return cdfs
 
 
-def _get_subplots(N:int):
+def _get_subplots(N:int, sharey:bool):
     if N < 5:
         M = 1
     else:
         M = int(np.sqrt(N))
         N = int(np.ceil(N/M))
     fig, axs = plt.subplots(N, M, figsize=(12,10), squeeze=False, sharex=True,
-                            sharey=True)
+                            sharey=sharey)
     return fig, axs
 
 
@@ -131,7 +131,7 @@ def make_plots(ns, cdfs, subplots:bool):
     cdfs.columns = [prettify(x) for x in cdfs.columns]
     cdfs.index.names = [prettify(x) for x in cdfs.index.names]
     if subplots:
-        fig, axs = _get_subplots(len(plot_cols))
+        fig, axs = _get_subplots(len(plot_cols), sharey=bool(ns.ylim))
     for n, col in enumerate(plot_cols):
         if subplots:
             ax = axs.ravel()[n]
@@ -147,9 +147,10 @@ def make_plots(ns, cdfs, subplots:bool):
         _plot_lines = df.plot(
             ax=ax, title=col, legend=None if subplots else ns.legend, alpha=.9)
         #  _legend = mpld3.plugins.InteractiveLegendPlugin(
-            #  *_plot_lines.get_legend_handles_labels())
+        #      *_plot_lines.get_legend_handles_labels())
         #  mpld3.plugins.connect(fig, _legend)
-
+        if ns.ylim:
+            ax.set_ylim(*ns.ylim)
         if not subplots:
             yield (fig, col)
     if subplots:
@@ -210,7 +211,7 @@ def _mode_1_and_4_get_perf_data_as_df(ns):
 
 
 def mode_1_plots(ns):
-    """Compare experiments.  
+    """Compare experiments.
     One plot for each metric, comparing the most recent result of each experiment
     """
     cdfs = _mode_1_and_4_get_perf_data_as_df(ns).reset_index('filename', drop=True)
@@ -295,12 +296,14 @@ def mode_3_plots(ns):
     timestamp = dt.datetime.utcnow().strftime('%Y%m%dT%H%M%S.%f')  # date plot was created.  nothing to do with timestamp column.
     plot_cols = [prettify(col) for col in cols]
     for col in plot_cols:
-        fig, ax = plt.subplots(1,1, figsize=(12,10))
+        fig, ax = plt.subplots(1,1, figsize=(12,10), sharey=bool(ns.ylim))
         kws = dict(x=prettify(ns.index_col), y=col, hue='Run Id', ax=ax, data=cdfs)
         sns.lineplot(**kws, ci=ns.mode3_ci or None, lw=2 if ns.mode3_ci == 0 else 1, legend=ns.legend)
         if ns.mode3_ci == 0:
             sns.lineplot(**kws, units="Filename", estimator=None, lw=1, alpha=.5)#, legend=False)
         ax.set_title(col)
+        if ns.ylim:
+            ax.set_ylim(*ns.ylim)
         if not ns.savefig: continue
         savefig_with_symlink(
             fig,
@@ -326,9 +329,9 @@ def _mode4_make_subplots(grp:pd.DataFrame, ns):
                 print(f'--mode4-groupby: No match found for "{col}" given regex: "{regex}"')
                 match_error = True
                 continue
-            try:
-                cg = match.group(int(grp_num))
-            except IndexError as e:
+            try:  # some validation to give a helpful error message
+                match.group(int(grp_num))
+            except IndexError:
                 msg = (
                     f'    match object: {match} '
                     f'\n    regex: "{regex}" '
@@ -344,7 +347,7 @@ def _mode4_make_subplots(grp:pd.DataFrame, ns):
                 ' more columns (see previously printed msgs)')
     assert len(colgrps) > 0, 'code bug'
     # choose nrows, ncols for the figure
-    fig, axs = _get_subplots(N=len(set(colgrps)))
+    fig, axs = _get_subplots(N=len(set(colgrps)), sharey=bool(ns.ylim))
     colgrps = np.array(colgrps)
     for ax, subplot_title in zip(axs.ravel(), sorted(set(colgrps))):
         cols = grp.columns[colgrps == subplot_title]
@@ -369,6 +372,8 @@ def _mode4_make_subplots(grp:pd.DataFrame, ns):
         if ns.legend:
             _handles, _labels = ax.get_legend_handles_labels()
         ax.get_legend().remove()
+        if ns.ylim:
+            ax.set_ylim(*ns.ylim)
         #  if not ns.legend:
             #  ax.get_legend().remove()
     if ns.legend:
@@ -420,6 +425,7 @@ def bap():
     A('--data-fp-regex', default=r'perf.*\.csv$', help=' ')
     #  A('--', nargs='?', default='one', const='two')
     A('--rolling-mean', '--rm', type=int, help='smoothing')
+    A('--ylim', nargs=2, type=float, help='Optional. Two numbers denoting the the y limit shared by all plots. Example: --ylim 0 1')
     A('--hdf-table-name', help='required if searching .h5 files')
     A('-c', '--col-regex', default='^(?!epoch|batch_idx|timestamp)', help='plot only columns matching regex.  By default, plot all except epoch and batch_idx.')
     A('--index-col', default='epoch', help=' ')
@@ -437,14 +443,14 @@ def bap():
     A('--mode4-savefig-dir', default='./results/plots/mode4', help=' ')
     A('--mode3-ci', type=float,
       help='confidence interval for error bars.  If 0, also shows all lines individually')
-    A('--mode1-subplots', action='store_true', help=dedent('''\
-        If defined, generate subplots instead of figures.'''))
+    A('--mode1-figures', dest='mode1_subplots', action='store_false', help=dedent('''\
+        If defined, generate individual figures instead of a figure with subplots.'''))
     A('--mode4-groupby', default="(Train|Val|Test) (.*):2", help=dedent('''\
         Optionally, make one sub-plot for each group of columns. To use, pass a
         regular expression extracting a string that we can group related
         columns followed by a ":N" where N is the one-indexed N-th capturing
         group to use.
-        Example:  `--mode4-subplots "(Train|Val|Test) (.*):2"
+        Example:  `--mode4-groupby "(Train|Val|Test) (.*):2"
           - this would place "train_loss" and "val_loss" into a subplot, and
             "train_acc" and "val_acc" in a different subplot.
         Note: This regex applies to the prettified text. The error messages
